# x^2 example with LinearRegression

import numpy as np
import matplotlib.pyplot as plt
from sklearn.linear_model import LinearRegression
from sklearn import metrics

np.random.seed(1)

x = np.random.random(20)
x = x.reshape(-1, 1)  # {ndarray: (20, 1)} 
y = 4.5 * x + 2.0

x2 = 4 * (x-0.5)
y2 = 4.5 * x2**2 - 3.0

reg = LinearRegression()

#
# Feature transformation: x^2 example with LinearRegression
from sklearn.preprocessing import PolynomialFeatures

# Generate a new feature matrix consisting of all polynomial combinations of
# the features with degree less than or equal to the specified degree.
# For example, if an input sample is two dimensional and of the form [a, b],
# the degree-2 polynomial features are [1, a, b, a^2, ab, b^2].
poly = PolynomialFeatures(3)

x2_poly = poly.fit_transform(x2)   # [1, x, x**2, x**3]  1 == bias

reg = LinearRegression()
reg.fit(x2_poly, y2)

print(f'We used y2 = 4.5 * x2**2 - 3.0\n' +
      f'LinearRegression got y = {reg.coef_} * x + {reg.intercept_}')


y2p_poly = reg.predict(x2_poly)
plt.plot(x2, y2, 'r+')
xi = np.arange(-2.5, 2.5, 0.1).reshape(-1, 1)
plt.plot(xi, reg.predict(poly.fit_transform(xi)), 'g-')
plt.show()
print(f'MSE: {metrics.mean_squared_error(y2, y2p_poly)}')
